import 'dart:io';

import 'package:fish_redux/fish_redux.dart';

import 'package:flutter/cupertino.dart' hide Action;
import 'package:flutter_livepush_demo/common/CommonUtils.dart';
import 'package:flutter_livepush_demo/constants/BizConstants.dart';
import 'package:flutter_livepush_demo/manager/push_config_manager.dart';
import 'package:flutter_livepush_demo/page/interactive/interactive_enrollment/interactive_enrollment_page.dart';
import 'package:flutter_livepush_demo/page/interactive/interactive_info/interactive_info_page.dart';
import 'package:flutter_livepush_plugin/live_base.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:flutter_livepush_demo/page/push_config/push_config_page.dart';
import 'package:flutter_livepush_demo/page/home/home_action.dart';
import 'package:flutter_livepush_demo/page/home/home_state.dart';

Effect<HomeState> buildEffect() {
  return combineEffects(<Object, Effect<HomeState>>{
    Lifecycle.initState: _initEffect,
    HomeAction.clickAliyunPrivacyPolicy: _clickAliyunPrivacyPolicy,
    HomeAction.clickCameraPush: _clickCameraPushAction,
    HomeAction.clickInteractiveLive: _clickInteractiveLiveAction,
    HomeAction.clickInteractivePK: _clickInteractivePKAction,
  });
}

Future<dynamic> _initEffect(Action action, Context<HomeState> ctx) async {
  CommomUtils.init();
  _getSDKVersion(action, ctx);
  DirType dirType;
  if (Platform.isAndroid) {
    dirType = DirType.externalFile;
  } else {
    dirType = DirType.document;
  }

  for (var element in musicList) {
    String path = element.path;
    if (path.isNotEmpty && !path.contains("http://")) {
      var split = path.split("/");
      CommomUtils.saveSystemDirWithAssetsResource(path, dirType, split[2]);
    }
  }
  for (var element in imgList) {
    if (element.isNotEmpty) {
      var split = element.split("/");
      CommomUtils.saveSystemDirWithAssetsResource(element, dirType, split[2]);
    }
  }
}

Future<dynamic> _getSDKVersion(Action action, Context<HomeState> ctx) async {
  String sdkVersion = await AlivcBase.getSdkVersion();
  ctx.dispatch(HomeActionCreator.getSDKVersion(sdkVersion));
}

Future<void> _clickAliyunPrivacyPolicy(
    Action action, Context<HomeState> ctx) async {
  final Uri url = Uri.parse(BizConstants.ALIYUN_PRIVACY_SERVICE_POLICY_URL);
  launchUrl(url);
}

///摄像头推流
Future<void> _clickCameraPushAction(
    Action action, Context<HomeState> ctx) async {
  bool hasPermission = await _checkPermission();
  if (hasPermission) {
    Navigator.of(ctx.context).push(
      CupertinoPageRoute(
          builder: (_) => ConfigPage().buildPage({
                pushConfigPageTypeKey: PushConfigPageType.basic,
              })),
    );
  }
}

///连麦互动
Future<void> _clickInteractiveLiveAction(
    Action action, Context<HomeState> ctx) async {
  jumpToInteractivePage(ctx.context, InteractiveTypeEnum.interactiveLive);
}

///PK互动
Future<void> _clickInteractivePKAction(
    Action action, Context<HomeState> ctx) async {
  jumpToInteractivePage(ctx.context, InteractiveTypeEnum.interactivePK);
}

///跳转到 PK互动/连麦互动/应用信息录入界面
void jumpToInteractivePage(
    BuildContext context, InteractiveTypeEnum type) async {
  bool hasPermission = await _checkPermission();
  if (hasPermission) {
    Map<String, dynamic>? checkResult = _checkAppInfo();
    if (checkResult != null) {
      //跳转页面时，map 增加当前类型，用于判断是连麦互动还是PK互动。
      checkResult[interactiveTypeKey] = type.name;
      Navigator.of(context).push(
        CupertinoPageRoute(
            builder: (_) => InteractiveEnrollmentPage().buildPage(checkResult)),
      );
    } else {
      Navigator.of(context).push(
        CupertinoPageRoute(
          builder: (_) => InteractiveInfoPage().buildPage(
            {interactiveTypeKey: type, interactiveFromHomePageKey: true},
          ),
        ),
      );
    }
  }
}

///检查 appId、appKey、playDomain，如果有则返回 map，没有则返回 null
Map<String, dynamic>? _checkAppInfo() {
  String appId = CommomUtils.getLocal(interactiveAppIdKey) as String? ?? "";
  String appKey = CommomUtils.getLocal(interactiveAppKeyKey) as String? ?? "";
  String playDomain =
      CommomUtils.getLocal(interactivePlayDomainKey) as String? ?? "";
  if (appId.isNotEmpty && appKey.isNotEmpty && playDomain.isNotEmpty) {
    var map = {
      interactiveAppIdKey: appId,
      interactiveAppKeyKey: appKey,
      interactivePlayDomainKey: playDomain,
    };
    return map;
  } else {
    return null;
  }
}

Future<bool> _checkPermission() async {
  if (await Permission.contacts.request().isGranted) {
    return true;
  }
  Map<Permission, PermissionStatus> statuses = await [
    Permission.microphone,
    Permission.camera,
  ].request();

  PermissionStatus checkPermissionStatue = _checkPermissionEnable(statuses);
  if (checkPermissionStatue == PermissionStatus.permanentlyDenied) {
    openAppSettings();
  }
  return checkPermissionStatue.isGranted;
}

PermissionStatus _checkPermissionEnable(
    Map<Permission, PermissionStatus> statuses) {
  for (Permission permission in statuses.keys) {
    PermissionStatus permissionStatus =
        statuses[permission] ?? PermissionStatus.granted;
    if (permissionStatus == PermissionStatus.denied) {
      return PermissionStatus.denied;
    } else if (permissionStatus == PermissionStatus.permanentlyDenied) {
      return PermissionStatus.permanentlyDenied;
    }
  }
  return PermissionStatus.granted;
}
